#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#
#   file: lib/Dist/Zilla/Plugin/Hook/AfterBuild.pm
#
#   This file is part of perl-Dist-Zilla-Plugin-Hook.
#   This file was generated with =tools::GenerateHooks.
#
#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#pod =head1 DESCRIPTION
#pod
#pod TODO
#pod
#pod =method after_build
#pod
#pod TODO
#pod
#pod =head1 SEE ALSO
#pod
#pod =for :list
#pod = L<Dist::Zilla::Plugin::Hook>
#pod = L<Dist::Zilla::Plugin::Hook:Manual>
#pod = L<Dist::Zilla::Plugin::Hook:ReadMe>
#pod = L<Dist::Zilla>
#pod = L<Dist::Zilla::Role::AfterBuild>
#pod
#pod =cut

package Dist::Zilla::Plugin::Hook::AfterBuild;

use Moose;

# ABSTRACT: TODO
our $VERSION = '0.002'; # VERSION

with 'Dist::Zilla::Role::Plugin';
with 'Dist::Zilla::Role::Hooker' => {
    -alias => { hook => 'after_build' },
};
with 'Dist::Zilla::Role::AfterBuild';

__PACKAGE__->meta->make_immutable();

1;

# end of file #

__END__

=pod

=encoding UTF-8

=head1 NAME

Dist::Zilla::Plugin::Hook::AfterBuild - TODO

=head1 VERSION

Version 0.002, released on 2015-07-23 22:08 UTC.

=head1 DESCRIPTION

TODO

=head1 OBJECT METHODS

=head2 after_build

TODO

=head1 SEE ALSO

=over 4

=item L<Dist::Zilla::Plugin::Hook>

=item L<Dist::Zilla::Plugin::Hook:Manual>

=item L<Dist::Zilla::Plugin::Hook:ReadMe>

=item L<Dist::Zilla>

=item L<Dist::Zilla::Role::AfterBuild>

=back

=head1 AUTHOR

Van de Bugger <van.de.bugger@gmail.com>

=cut
