use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'Build.PL',
    'COPYING',
    'Changes',
    'GPLv3',
    'META.json',
    'META.yml',
    'README',
    'VERSION',
    'ex/Description/README.pod',
    'ex/Description/dist.ini',
    'ex/TemplateVariables/BUGS.pod',
    'ex/TemplateVariables/README.pod',
    'ex/TemplateVariables/dist.ini',
    'ex/UnwantedDependencies/README.pod',
    'ex/UnwantedDependencies/dist.ini',
    'ex/VersionHandling/README.pod',
    'ex/VersionHandling/VERSION',
    'ex/VersionHandling/dist.ini',
    'lib/Dist/Zilla/Plugin/Hook.pm',
    'lib/Dist/Zilla/Plugin/Hook/AfterBuild.pm',
    'lib/Dist/Zilla/Plugin/Hook/AfterMint.pm',
    'lib/Dist/Zilla/Plugin/Hook/AfterRelease.pm',
    'lib/Dist/Zilla/Plugin/Hook/BeforeArchive.pm',
    'lib/Dist/Zilla/Plugin/Hook/BeforeBuild.pm',
    'lib/Dist/Zilla/Plugin/Hook/BeforeMint.pm',
    'lib/Dist/Zilla/Plugin/Hook/BeforeRelease.pm',
    'lib/Dist/Zilla/Plugin/Hook/FileGatherer.pm',
    'lib/Dist/Zilla/Plugin/Hook/FileMunger.pm',
    'lib/Dist/Zilla/Plugin/Hook/FilePruner.pm',
    'lib/Dist/Zilla/Plugin/Hook/Init.pm',
    'lib/Dist/Zilla/Plugin/Hook/InstallTool.pm',
    'lib/Dist/Zilla/Plugin/Hook/LicenseProvider.pm',
    'lib/Dist/Zilla/Plugin/Hook/Manual.pod',
    'lib/Dist/Zilla/Plugin/Hook/MetaProvider.pm',
    'lib/Dist/Zilla/Plugin/Hook/NameProvider.pm',
    'lib/Dist/Zilla/Plugin/Hook/ReadMe.pod',
    'lib/Dist/Zilla/Plugin/Hook/ReleaseStatusProvider.pm',
    'lib/Dist/Zilla/Plugin/Hook/VersionProvider.pm',
    'lib/Dist/Zilla/Role/Hooker.pm',
    't/DistZillaTester.pm',
    't/HookTester.pm',
    't/PluginTester.pm',
    't/hook.t',
    'xt/aspell.en.pws',
    'xt/author/critic.t',
    'xt/author/eol.t',
    'xt/author/no-tabs.t',
    'xt/author/pod-spell.t',
    'xt/author/test-version.t',
    'xt/examples.t',
    'xt/perlcritic.ini',
    'xt/release/cpan-changes.t',
    'xt/release/dist-manifest.t',
    'xt/release/fixme.t',
    'xt/release/kwalitee.t',
    'xt/release/meta-json.t',
    'xt/release/minimum-version.t',
    'xt/release/mojibake.t',
    'xt/release/new-version.t',
    'xt/release/pod-coverage.t',
    'xt/release/pod-no404s.t',
    'xt/release/pod-syntax.t'
);

notabs_ok($_) foreach @files;
done_testing;
